#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var parser_1 = require("../parser");
var validator_1 = __importDefault(require("../validator"));
var normalizer_1 = __importDefault(require("../normalizer"));
var options_1 = require("./options");
var process = require('process');
var parseCommand = function (options) {
    try {
        var strictSyntax = options.options.includes('strict');
        var ast = (0, parser_1.parse)(options.expression, strictSyntax);
        var result = {
            expression: ast,
            errors: (0, validator_1.default)(options.expression).errors
        };
        return JSON.stringify(result, null, 2);
    }
    catch (error) {
        return JSON.stringify({ expression: undefined, errors: [error.message] }, null, 2);
    }
};
var validateCommand = function (options) {
    return JSON.stringify((0, validator_1.default)(options.expression), null, 2);
};
var normalizeCommand = function (options) {
    return (0, normalizer_1.default)(options.expression);
};
var mapOptionsToCommand = function (options) {
    if (options.command === 'normalize') {
        return normalizeCommand;
    }
    else if (options.command === 'validate') {
        return validateCommand;
    }
    else {
        return parseCommand;
    }
};
var cliRunner = function (args) {
    var cliOptions = (0, options_1.parseCLIOptions)(args);
    if (cliOptions.expression.trim().length === 0) {
        console.log((0, options_1.usage)('spdx'));
    }
    else {
        var command = mapOptionsToCommand(cliOptions);
        console.log(command.call(command, cliOptions));
    }
};
function runCLI() {
    cliRunner(process.argv.slice(2));
}
runCLI();
