"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseCLIOptions = exports.usage = void 0;
var commands = [
    {
        name: 'parse',
        flags: ['-p', '--parse'],
        commandOptions: [
            {
                name: 'strict',
                flags: ['-s', '--strict'],
                description: 'Use strict parsing rules'
            }
        ],
        description: 'Parse an expression into a structured object'
    },
    {
        name: 'validate',
        flags: ['-v', '--validate'],
        description: 'Validate the semantic correctness of an expression'
    },
    {
        name: 'normalize',
        flags: ['-n', '--normalize', '--normalise'],
        description: 'Normalize an expression into its canonical form'
    },
];
var longestFlagCombination = Math.max.apply(Math, __spreadArray([], __read(commands.map(function (cmd) {
    var optionsLength = Math.max.apply(Math, __spreadArray([], __read((cmd.commandOptions || []).map(function (opt) { return opt.flags.join(', ').length; })), false));
    var commandLength = cmd.flags.join(', ').length;
    return Math.max(optionsLength, commandLength);
})), false));
var usage = function (nameOfExecutable) {
    var rows = [];
    rows.push("Usage:  ".concat(nameOfExecutable, " [command] [options] EXPRESSION"));
    rows.push('');
    rows.push('Commands:');
    commands.forEach(function (cmd) {
        rows.push("\n".concat(cmd.flags.join(', ').padEnd(longestFlagCombination + 4), "\t").concat(cmd.description));
        if (cmd.commandOptions && cmd.commandOptions.length > 0) {
            rows.push('\n  Options:');
            cmd.commandOptions.forEach(function (opt) {
                rows.push("  ".concat(opt.flags.join(', ').padEnd(longestFlagCombination), "\t").concat(opt.description));
            });
        }
    });
    rows.push('');
    return rows.join('\n');
};
exports.usage = usage;
var parseCLIOptions = function (args) {
    var params = __spreadArray([], __read(args), false);
    var selectedCommand = commands[0];
    var options = {
        command: selectedCommand.name,
        options: [],
        expression: args.join(' '),
    };
    // First, check if the command was provided explicitly:
    var explicitCommand = commands.find(function (cmd) { return cmd.flags.includes(params[0]); });
    if (explicitCommand) {
        selectedCommand = explicitCommand;
        options.command = selectedCommand.name;
        params = params.slice(1);
    }
    // Then, consume any command-specific options:
    while (params[0] && params[0].startsWith('-')) {
        var commandOption = (selectedCommand.commandOptions || []).find(function (opt) { return opt.flags.includes(params[0]); });
        if (commandOption) {
            options.options.push(commandOption.name);
            params = params.slice(1);
        }
        else {
            throw new Error("Option ".concat(params[0], " is not supported by the ").concat(selectedCommand.name, " command."));
        }
    }
    options.expression = params.join(' ');
    return options;
};
exports.parseCLIOptions = parseCLIOptions;
exports.default = exports.parseCLIOptions;
