"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isKnownExceptionId = exports.isKnownLicenseId = exports.correctExceptionId = exports.correctLicenseId = exports.exceptions = exports.licenses = void 0;
var spdx_correct_1 = __importDefault(require("spdx-correct"));
var data_1 = require("./data");
Object.defineProperty(exports, "licenses", { enumerable: true, get: function () { return data_1.licenses; } });
Object.defineProperty(exports, "exceptions", { enumerable: true, get: function () { return data_1.exceptions; } });
var createMapOfIds = function (ids) {
    var mutilateId = function (id) { return id.toUpperCase().replace(/[^0-9a-zA-Z\+]/g, ''); };
    var listOfOfficialIds = ids;
    var mapOfLowercaseIds = new Map();
    var mapOfMutilatedIds = new Map();
    listOfOfficialIds.forEach(function (id) {
        mapOfLowercaseIds.set(id.toLowerCase(), id);
        mapOfMutilatedIds.set(mutilateId(id), id);
    });
    var getExactMatch = function (id) { return listOfOfficialIds.find(function (x) { return x === id; }); };
    var getCaseInsensitiveMatch = function (id) {
        var lowercaseId = id.toLowerCase();
        var variationsToTest = [
            lowercaseId,
            lowercaseId.replace(/\s+/g, '-').replace(/(\d+)$/, '$1.0')
        ];
        return variationsToTest.map(function (x) { return mapOfLowercaseIds.get(x); }).filter(function (m) { return !!m; })[0];
    };
    var getFuzzyMatch = function (id) { return mapOfMutilatedIds.get(mutilateId(id)); };
    var get = function (id) {
        return getExactMatch(id) || getCaseInsensitiveMatch(id) || getFuzzyMatch(id);
    };
    return { get: get };
};
var mapOfKnownLicenses = createMapOfIds(data_1.licenses.licenses.map(function (lic) { return lic.licenseId; }));
var mapOfKnownExceptions = createMapOfIds(data_1.exceptions.exceptions.map(function (e) { return e.licenseExceptionId; }));
var aliasesForLicenseIds = (function () {
    var mapOfAliases = new Map();
    // Aliases for the BSD-2-Clause license:
    mapOfAliases.set('freebsd', 'BSD-2-Clause');
    mapOfAliases.set('freebsd license', 'BSD-2-Clause');
    mapOfAliases.set('freebsd license', 'BSD-2-Clause');
    mapOfAliases.set('simplified bsd license', 'BSD-2-Clause');
    // Aliases for the BSD-3-Clause license:
    mapOfAliases.set('new bsd license', 'BSD-3-Clause');
    mapOfAliases.set('modified bsd license', 'BSD-3-Clause');
    // Aliases for the 0BSD license:
    mapOfAliases.set('zero-clause bsd', '0BSD');
    mapOfAliases.set('free public license', '0BSD');
    mapOfAliases.set('free public license 1.0.0', '0BSD');
    mapOfAliases.set('free public license 1.0', '0BSD');
    mapOfAliases.set('bsd0', '0BSD');
    // Aliases for the Apache-1.1 license:
    mapOfAliases.set('apache1.1', 'Apache-1.1');
    mapOfAliases.set('apache-1.1', 'Apache-1.1');
    mapOfAliases.set('apache software license', 'Apache-1.1');
    mapOfAliases.set('apache software license 1.1', 'Apache-1.1');
    mapOfAliases.set('apache software license version 1.1', 'Apache-1.1');
    return mapOfAliases;
})();
var aliasesForExceptions = (function () {
    var mapOfAliases = new Map();
    mapOfAliases.set('qwt license 1.0', 'Qwt-exception-1.0');
    return mapOfAliases;
})();
var mapLicenseAlias = function (alias) {
    return aliasesForLicenseIds.get(alias.toLowerCase());
};
var mapLicenseId = function (id) {
    return mapOfKnownLicenses.get(id.toLowerCase());
};
var fixLicenseId = function (id) {
    // Don't try to correct license identifiers that are syntactically legit on the
    // surface and have an explicit version number such as "3.0" or "2.1":
    if (id.match(/([a-zA-Z_0-9]+(\-[a-zA-Z_0-9]+)*)\-\d+(\.\d+)+(\-[a-zA-Z_0-9]+)*/)) {
        return id;
    }
    // Use `spdx-correct` to try and correct identifiers that we haven't fixed already:
    return (0, spdx_correct_1.default)(id, { upgrade: true });
};
var mapExceptionAlias = function (alias) {
    return aliasesForExceptions.get(alias.toLowerCase());
};
var mapExceptionId = function (id) {
    return mapOfKnownExceptions.get(id.toLowerCase());
};
var fixExceptionid = function (id) {
    var variations = [
        id.replace(/\s+/, '-'),
        id.replace(/\s+version\s+/i, ' '),
        id.replace(/(\d+)$/, '$1.0')
    ];
    var matchedIds = variations.map(mapExceptionId);
    return matchedIds.filter(function (matchedId) { return !!matchedId; })[0];
};
var mapExceptionIdByAssociatedLicense = function (id, associatedLicense) {
    var expandLicenses = function (identifiers) {
        var expandedList = __spreadArray([], __read(identifiers), false);
        identifiers.forEach(function (id) {
            if (id.match(/[^\+]\+$/)) {
                expandedList.push(id.replace(/\+$/, '-or-later'));
                expandedList.push(id.replace(/\+$/, ''));
            }
            else if (id.match(/-or-later$/)) {
                expandedList.push(id.replace(/-or-later$/, ''));
            }
            else if (id.match(/-only$/)) {
                expandedList.push(id.replace(/-only$/, ''));
            }
            else if (id.match(/[AL]?GPL\-\d\.\d$/)) {
                expandedList.push("".concat(id, "-only"));
                expandedList.push("".concat(id, "-or-later"));
            }
        });
        return expandedList;
    };
    if (associatedLicense) {
        var possibleMatches = data_1.exceptions.exceptions.filter(function (e) {
            var hasRelation = function () {
                var relatedLicenses = expandLicenses(e.relatedLicenses);
                return relatedLicenses.length === 0 || relatedLicenses.includes(associatedLicense);
            };
            var looksSimilar = function () {
                return e.licenseExceptionId.toLowerCase().startsWith(id.toLowerCase());
            };
            return looksSimilar() && hasRelation();
        }).map(function (e) { return e.licenseExceptionId; }).sort();
        if (possibleMatches.length > 0) {
            return possibleMatches[0];
        }
    }
    return undefined;
};
/**
 * Fix the given license identifier, if possible, or return it unchanged.
 *
 * @param identifier SPDX license identifier to "fix" if possible
 * @returns A corrected SPDX license identifier or the original, if no suitable "fix" was found
 */
function correctLicenseId(identifier) {
    var mapLicense = function (id) { return mapLicenseId(id) || mapLicenseAlias(id); };
    // The `spdx-correct` package does not have fixes for everything (e.g. a common misspelling for the
    // zero-clause variant of the BSD license or for names such as "The Simplified BSD license") so we'll
    // check for a list of known aliases ourselves:
    var applyAliases = function (id) {
        var lowercaseId = id.replace(/^the /i, '');
        return mapLicense(lowercaseId) || mapLicense("the ".concat(lowercaseId)) || id;
    };
    // The `spdx-correct` package will coerce "GPL-3.0" into "GPL-3.0-or-later", although
    // "GPL-3.0-only" would be more true to the original intent:
    var applyGPLFixes = function (id) {
        if (id.toUpperCase() === 'GPL')
            return mapLicense('GPL-3.0-only') || id;
        if (id.toUpperCase() === 'LGPL')
            return mapLicense('LGPL-3.0-only') || id;
        if (id.toUpperCase() === 'AGPL')
            return mapLicense('AGPL-3.0-only') || id;
        if (mapLicense(id)) {
            return mapLicense("".concat(id, "-only")) || id;
        }
        if (id.match(/^([AL]?GPL)([\-vV]?)(\d)(\+?)$/)) {
            return applyGPLFixes(id.replace(/^([AL]?GPL)([\-vV]?)(\d)(\+?)$/, '$1-$3.0$4'));
        }
        if (id.match(/^([AL]?GPL[\-vV]?\d)(\+)$/)) {
            var candidateId = id.replace(/([AL]?GPL[\-vV]?\d)(\+)$/, '$1.0-or-later');
            console.log("candidateId: ".concat(id, " => ").concat(candidateId, " => ").concat(mapLicense(candidateId)));
            return mapLicense(candidateId) || id;
        }
        return id;
    };
    // Expand the "+" syntax to "-or-later" if one exists:
    var expandPlus = function (id) {
        if (id.match(/[^\+]\+$/)) {
            var orLaterId = id.replace(/\+$/, '-or-later');
            return mapLicense(orLaterId) || id;
        }
        return id;
    };
    var id = expandPlus(applyGPLFixes(applyAliases(identifier)));
    return mapLicense(id) || fixLicenseId(id) || id;
}
exports.correctLicenseId = correctLicenseId;
/**
 * Fix the given exception identifier, if possible, or return it unchanged.
 *
 * @param identifier SPDX exception identifier to "fix" if possible
 * @returns A corrected SPDX exception identifier or the original, if no suitable "fix" was found
 */
function correctExceptionId(identifier, associatedLicense) {
    var removeExtras = function (id) {
        return id.replace(/^the\s+/i, '').replace(/\s+/, ' ');
    };
    var applyAliases = function (id) {
        var lowercaseId = id.toLowerCase();
        var lowercaseIdWithoutWordVersion = lowercaseId.replace(/\s+version\s+(\d+)/, ' $1');
        return mapExceptionAlias(lowercaseId) || mapExceptionAlias(lowercaseIdWithoutWordVersion) || id;
    };
    var id = applyAliases(removeExtras(identifier));
    return mapExceptionId(id) || fixExceptionid(id) || mapExceptionIdByAssociatedLicense(id, associatedLicense) || id;
}
exports.correctExceptionId = correctExceptionId;
function isKnownLicenseId(id) {
    return !!mapLicenseId(id);
}
exports.isKnownLicenseId = isKnownLicenseId;
function isKnownExceptionId(id) {
    return !!mapExceptionId(id);
}
exports.isKnownExceptionId = isKnownExceptionId;
