"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("assert");
var parser_1 = require("../parser");
var sortKey = function (element) {
    if (element.conjunction) {
        var conj = element;
        var children = [sortKey(conj.left), sortKey(conj.right)].sort();
        return "".concat(conj.conjunction, "(").concat(children.join(','), ")");
    }
    else if (element.licenseRef) {
        return '_1_' + renderExpression(element);
    }
    else if (element.license) {
        return '_0_' + renderExpression(element);
    }
    /* istanbul ignore next */ (0, assert_1.fail)('Unexpected input for sortKey() => ' + JSON.stringify(element));
};
var compareElements = function (element1, element2) {
    var key1 = sortKey(element1);
    var key2 = sortKey(element2);
    return key1.localeCompare(key2);
};
var sortElement = function (element) {
    if (element.conjunction) {
        return sortConjunction(element);
    }
    else {
        return element;
    }
};
var sortConjunction = function (element) {
    if (compareElements(element.left, element.right) > 0) {
        return __assign(__assign({}, element), { left: sortElement(element.right), right: sortElement(element.left) });
    }
    else {
        return __assign(__assign({}, element), { left: sortElement(element.left), right: sortElement(element.right) });
    }
};
var renderLicenseInfoExpression = function (expression) {
    if (expression.exception) {
        return "".concat(expression.license, " WITH ").concat(expression.exception);
    }
    else {
        return expression.license;
    }
};
var renderLicenseRefExpression = function (expression) {
    if (expression.documentRef) {
        return "".concat(expression.documentRef, ":").concat(expression.licenseRef);
    }
    else {
        return expression.licenseRef;
    }
};
var renderConjunctionInfoExpression = function (expression, options) {
    var separator = expression.conjunction.toUpperCase();
    var childOptions = __assign(__assign({}, options), { inner: true });
    var left = renderExpression(expression.left, childOptions);
    var right = renderExpression(expression.right, childOptions);
    var prefix = options.inner ? '(' : '';
    var suffix = options.inner ? ')' : '';
    return prefix + left + ' ' + separator + ' ' + right + suffix;
};
var defaultOptions = { inner: false };
var renderExpression = function (expression, options) {
    var effectiveOptions = __assign(__assign({}, defaultOptions), options);
    if (expression.license) {
        return renderLicenseInfoExpression(expression);
    }
    else if (expression.licenseRef) {
        return renderLicenseRefExpression(expression);
    }
    else if (expression.conjunction) {
        return renderConjunctionInfoExpression(expression, effectiveOptions);
    }
    /* istanbul ignore next */ (0, assert_1.fail)('Unexpected input for renderExpression() => ' + JSON.stringify(expression));
};
/**
 * Renders a normalized string representation of an SPDX expression.
 *
 * @param spdxExpression The SPDX expression to render in a normalized form.
 * @returns Normalized string representation of the given SPDX expression.
 */
function normalize(spdxExpression) {
    try {
        return renderExpression(sortElement((0, parser_1.parse)(spdxExpression)));
    }
    catch (error) {
        return spdxExpression;
    }
}
exports.default = normalize;
