"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
var assert_1 = __importDefault(require("assert"));
var StrictParser = __importStar(require("../codegen/parser_strict"));
var licenses_1 = require("../licenses");
var extractErrorMessage = function (tree) {
    if (tree.errs.length === 0)
        return undefined;
    return tree.errs.map(function (element) { return element.toString(); }).join('\n');
};
function parse(input) {
    var _a;
    var reduceNode = function (node) {
        if (!node) {
            return undefined;
        }
        var convertLicenseIdExpression = function (node) {
            var correctedLicenseId = (0, licenses_1.correctLicenseId)(node.license.license);
            if (!!node.exception) {
                var correctedExceptionId = (0, licenses_1.correctExceptionId)(node.exception.exception);
                return { license: correctedLicenseId, exception: correctedExceptionId };
            }
            else {
                return { license: correctedLicenseId };
            }
        };
        var converLicenseRefExpression = function (node) {
            var extractDocumentRef = function (node) {
                if (node.document_ref && node.document_ref.prefix) {
                    return [node.document_ref.prefix, node.document_ref.value].join('');
                }
                return undefined;
            };
            var extractLicenseRef = function (node) {
                return [node.license_ref.prefix, node.license_ref.value].join('');
            };
            return {
                documentRef: extractDocumentRef(node),
                licenseRef: extractLicenseRef(node),
            };
        };
        if (node.kind === 'license_id_expression') {
            return convertLicenseIdExpression(node);
        }
        else if (node.kind === 'wrapped_expression') {
            return reduceNode(node.value);
        }
        else if (node.kind === 'or_expression') {
            var compound = {
                conjunction: 'or',
                left: reduceNode(node.left),
                right: reduceNode(node.right)
            };
            return compound;
        }
        else if (node.kind === 'and_expression') {
            var compound = {
                conjunction: 'and',
                left: reduceNode(node.left),
                right: reduceNode(node.right)
            };
            return compound;
        }
        else if (node.kind.startsWith('license_ref_expression')) {
            return converLicenseRefExpression(node);
        }
        /* istanbul ignore next */ assert_1.default.fail("reduceNode() did not recognize input: ".concat(JSON.stringify(node)));
    };
    var p = new StrictParser.Parser(input);
    var tree = p.parse();
    var expression = reduceNode((_a = tree.ast) === null || _a === void 0 ? void 0 : _a.value);
    return { parse: tree, expression: expression || undefined, error: extractErrorMessage(tree) };
}
exports.parse = parse;
